from getpass import getpass
import os
import json
import subprocess
import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
import pwd
import logging
import gi
# gi.require_version('Gio', '2.0')
from gi.repository import GLib

# 获取当前用户的主目录
user_home = os.path.expanduser("~")

# 设置日志文件路径
log_file_path = os.path.join(user_home, 'nfs_domain_user_settings.log')

# 设置日志记录
logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(message)s',
                    handlers=[
                        logging.FileHandler(log_file_path),
                        logging.StreamHandler()
                    ])
logger = logging.getLogger(__name__)


class UserSettingsService(dbus.service.Object):

    def __init__(self, bus_name, object_path):
        super().__init__(bus_name, object_path)
        # 监听系统级信号
        system_bus = dbus.SystemBus()
        system_bus.add_signal_receiver(
            self.handle_signal_processUserSettings,
            dbus_interface="org.freedesktop.FleetCommanderClient",
            signal_name="processUserSettings"
        )

    def handle_signal_processUserSettings(self,uid):
        logger.info(f"Receive a signal form processUserSettings ,uid is {uid}")
        if uid == os.getuid():
            self.SetUserSettings(uid)

    def get_realm_list(self):
        try:
            result = subprocess.run(['realm', 'list'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            # 检查命令是否成功执行
            if result.returncode != 0:
                logger.error("Error executing 'realm list':", result.stderr)
                return None
            # 解析输出
            output = result.stdout.strip()
            logger.debug(output)
            if output:
                # 提取域名
                domains = []
                for line in output.split('\n'):
                    if line.startswith(' '):  # 过滤掉缩进行，只保留顶级域名行
                        continue
                    domain = line.strip()
                    domains.append(domain)
                return domains
            else:
                return None

        except Exception as e:
            logger.error(f"An error occurred: {e}")
            return None

    @dbus.service.method('com.nfs.domain.UserSettings', in_signature='i',out_signature='s')
    def SetUserSettings(self, uid):

        domains = self.get_realm_list()
        if domains:
            logger.info("The system is joined to the following domain(s):")
            for domain in domains:
                logger.info(domain)
            domain_name = domains[0]
        else:
            logger.info("The system is not joined to any domain.return")
            return "The system is not joined to any domain"

        if uid == 0 :
            uid = os.getuid()
            if uid < 10000:
                 logger.info("The user is not a domain user")
                 return "The user is not a domain user"
        try:
            # 将 UID 转换成用户名
            username = pwd.getpwuid(uid).pw_name
            logger.info(f"Converted UID {uid} to username {username}")
        except KeyError:
            error_message = f"User with UID {uid} does not exist"
            logger.error(error_message)
            return error_message

        directory = f"/var/lib/sss/deskprofile/{domain_name}/{username}/"
        if not os.path.isdir(directory):
            error_message = f"Directory {directory} does not exist"
            logger.error(error_message)
            return error_message

        for filename in os.listdir(directory):
            if filename.endswith('.json'):
                file_path = os.path.join(directory, filename)
                logger.info(f"Processing file: {file_path}")
                with open(file_path, 'r') as json_file:
                    try:
                        data = json.load(json_file)
                        for key, value in data.items():
                            if key == "org.gnome.gsettings":
                                for setting in value: 
                                    self.set_gsettings(setting)
                    except json.JSONDecodeError as e:
                        error_message = f"Error parsing {file_path}: {e}"
                        logger.error(error_message)
                        return error_message

        success_message = "Settings updated successfully"
        logger.info(success_message)
        return success_message

    def set_gsettings(self, setting):
        try:
            schema = setting.get('schema')
            key = setting.get('key')
            value = setting.get('value')
            signature = setting.get('signature')

            if schema and key and value is not None:
                # 将值转换成字符串形式以便传递给 gsettings 命令
                value_str = str(value)
                subprocess.run(['gsettings', 'set', schema, key, value_str], check=True)
                logger.info(f"Set gsettings for schema: {schema}, key: {key}, value: {value_str}")
        except subprocess.CalledProcessError as e:
            logger.error(f"Error setting gsettings: {e}")

def main():
    DBusGMainLoop(set_as_default=True)
    bus = dbus.SessionBus()
    bus_name = dbus.service.BusName('com.nfs.domain.UserSettings', bus)
    service = UserSettingsService(bus_name, '/com/nfs/domain/UserSettings')
    loop = GLib.MainLoop()
    logger.info("Running User Settings D-Bus Service...")
    loop.run()

if __name__ == "__main__":
    main()
